@@ -791,8 +791,8 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
                   dir_to_create );
       return MS_FAILURE;
     }
-  } else
-    /* handled later */;
+  }
+  /*  else handled later */
 
   /* -------------------------------------------------------------------- */
   /*      Setup the full datasource name.                                 */
@@ -1088,13 +1088,21 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
       /*
       ** Read the shape.
       */
-      status = msLayerGetShape(layer, &resultshape, &(layer->resultcache->results[i]));
-      if(status != MS_SUCCESS) {
-        OGR_DS_Destroy( hDS );
-        msOGRCleanupDS( datasource_name );
-        msGMLFreeItems(item_list);
-        msFreeShape(&resultshape);
-        return status;
+      if( layer->resultcache->results[i].shape )
+      {
+          /* msDebug("Using cached shape %ld\n", layer->resultcache->results[i].shapeindex); */
+          msCopyShape(layer->resultcache->results[i].shape, &resultshape);
+      }
+      else
+      {
+        status = msLayerGetShape(layer, &resultshape, &(layer->resultcache->results[i]));
+        if(status != MS_SUCCESS) {
+            OGR_DS_Destroy( hDS );
+            msOGRCleanupDS( datasource_name );
+            msGMLFreeItems(item_list);
+            msFreeShape(&resultshape);
+            return status;
+        }
       }
 
       /*
